@@ -5,13 +5,30 @@
   Copyright 2007-2009 Y Giridhar Appaji Nag
   Copyright 2008-2009 Philipp Hagemeister
   Copyright 2015-2016 Joao Eriberto Mota Filho
+  Copyright 2016      Denis Denisov
   Copyright 2016      Ivan Gimenez
+  Copyright 2016      Sjjad Hashemian
+  Copyright 2016      Stephen Thirlwall
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations including
+  the two.
+
+  You must obey the GNU General Public License in all respects for all
+  of the code used other than OpenSSL. If you modify file(s) with this
+  exception, you may extend this exception to your version of the
+  file(s), but you are not obligated to do so. If you do not wish to do
+  so, delete this exception statement from your version. If you delete
+  this exception statement from all source files in the program, then
+  also delete it here.
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -132,6 +149,10 @@ axel_t *axel_new( conf_t *conf, int count, void *url )
 	if( strchr( axel->filename, '*' ) || strchr( axel->filename, '?' ) )
 		strncpy( axel->filename, axel->conn[0].file, MAX_STRING );
 
+	if( *axel->conn[0].output_filename != 0 ) {
+		strncpy( axel->filename, axel->conn[0].output_filename, MAX_STRING );
+	}
+
 	return( axel );
 }
 
@@ -161,19 +182,37 @@ int axel_open( axel_t *axel )
 	}
 	else if( ( fd = open( buffer, O_RDONLY ) ) != -1 )
 	{
+		int old_format = 0;
+		off_t stsize = lseek(fd,0,SEEK_END);
+		lseek(fd,0,SEEK_SET);
+
 		nread = read( fd, &axel->conf->num_connections, sizeof( axel->conf->num_connections ) );
 		assert( nread == sizeof( axel->conf->num_connections ) );
 
+		if(stsize < sizeof( axel->conf->num_connections ) + sizeof( axel->bytes_done )
+			+ 2 * axel->conf->num_connections * sizeof( axel->conn[i].currentbyte ))
+		{
+#ifdef DEBUG
+			printf( "State file has old format.\n" );
+#endif
+			old_format = 1;
+		}
+
 		axel->conn = realloc( axel->conn, sizeof( conn_t ) * axel->conf->num_connections );
 		memset( axel->conn + 1, 0, sizeof( conn_t ) * ( axel->conf->num_connections - 1 ) );
 
-		axel_divide( axel );
+		if(old_format)
+			axel_divide( axel );
 
 		nread = read( fd, &axel->bytes_done, sizeof( axel->bytes_done ) );
 		assert( nread == sizeof( axel->bytes_done ) );
 		for( i = 0; i < axel->conf->num_connections; i ++ ) {
 			nread = read( fd, &axel->conn[i].currentbyte, sizeof( axel->conn[i].currentbyte ) );
 			assert( nread == sizeof( axel->conn[i].currentbyte ) );
+			if( !old_format ) {
+				nread = read( fd, &axel->conn[i].lastbyte, sizeof( axel->conn[i].lastbyte ) );
+				assert( nread == sizeof( axel->conn[i].lastbyte ) );
+			}
 		}
 
 		axel_message( axel, _("State file found: %lld bytes downloaded, %lld to go."),
@@ -225,6 +264,35 @@ int axel_open( axel_t *axel )
 	return( 1 );
 }
 
+void reactivate_connection(axel_t *axel, int thread)
+{
+	long long int max_remaining = 0,remaining;
+	int j, idx = -1;
+
+	if(axel->conn[thread].enabled || axel->conn[thread].currentbyte <= axel->conn[thread].lastbyte)
+		return;
+	/* find some more work to do */
+	for( j = 0; j < axel->conf->num_connections; j ++ )
+	{
+		remaining = axel->conn[j].lastbyte - axel->conn[j].currentbyte + 1;
+		if(remaining > max_remaining)
+		{
+			max_remaining = remaining;
+			idx = j;
+		}
+	}
+	/* do not reactivate for less than 100KB */
+	if(max_remaining >= 100 * 1024 && idx != -1)
+	{
+#ifdef DEBUG
+		printf("\nReactivate connection %d\n",thread);
+#endif
+		axel->conn[thread].lastbyte = axel->conn[idx].lastbyte;
+		axel->conn[idx].lastbyte = axel->conn[idx].currentbyte + max_remaining/2;
+		axel->conn[thread].currentbyte = axel->conn[idx].lastbyte + 1;
+	}
+}
+
 /* Start downloading */
 void axel_start( axel_t *axel )
 {
@@ -245,6 +313,12 @@ void axel_start( axel_t *axel )
 	if( axel->conf->verbose > 0 )
 		axel_message( axel, _("Starting download") );
 
+	for( i = 0; i < axel->conf->num_connections; i ++ )
+	if( axel->conn[i].currentbyte > axel->conn[i].lastbyte )
+	{
+		reactivate_connection(axel, i);
+	}
+
 	for( i = 0; i < axel->conf->num_connections; i ++ )
 	if( axel->conn[i].currentbyte <= axel->conn[i].lastbyte )
 	{
@@ -291,9 +365,10 @@ void axel_do( axel_t *axel )
 	hifd = 0;
 	for( i = 0; i < axel->conf->num_connections; i ++ )
 	{
-		if( axel->conn[i].enabled )
-			FD_SET( axel->conn[i].fd, fds );
-		hifd = max( hifd, axel->conn[i].fd );
+		if( axel->conn[i].enabled ) {
+			FD_SET( axel->conn[i].tcp->fd, fds );
+			hifd = max( hifd, axel->conn[i].tcp->fd );
+		}
 	}
 	if( hifd == 0 )
 	{
@@ -317,10 +392,10 @@ void axel_do( axel_t *axel )
 	/* Handle connections which need attention */
 	for( i = 0; i < axel->conf->num_connections; i ++ )
 	if( axel->conn[i].enabled ) {
-	if( FD_ISSET( axel->conn[i].fd, fds ) )
+	if( FD_ISSET( axel->conn[i].tcp->fd, fds ) )
 	{
 		axel->conn[i].last_transfer = gettime();
-		size = read( axel->conn[i].fd, buffer, axel->conf->buffer_size );
+		size = tcp_read( axel->conn[i].tcp, buffer, axel->conf->buffer_size );
 		if( size == -1 )
 		{
 			if( axel->conf->verbose )
@@ -352,6 +427,7 @@ void axel_do( axel_t *axel )
 			}
 			axel->conn[i].enabled = 0;
 			conn_disconnect( &axel->conn[i] );
+			reactivate_connection(axel,i);
 			continue;
 		}
 		/* remaining == Bytes to go */
@@ -378,6 +454,8 @@ void axel_do( axel_t *axel )
 		}
 		axel->conn[i].currentbyte += size;
 		axel->bytes_done += size;
+		if( remaining == size )
+			reactivate_connection(axel,i);
 	}
 	else
 	{
@@ -525,14 +603,16 @@ void save_state( axel_t *axel )
 	}
 
         nwrite = write( fd, &axel->conf->num_connections, sizeof( axel->conf->num_connections ) );
-	assert( nwrite == sizeof( axel->conf->num_connections ) ); 
+	assert( nwrite == sizeof( axel->conf->num_connections ) );
 
 	nwrite = write( fd, &axel->bytes_done, sizeof( axel->bytes_done ) );
-	assert( nwrite == sizeof( axel->bytes_done ) ); 
+	assert( nwrite == sizeof( axel->bytes_done ) );
 
 	for( i = 0; i < axel->conf->num_connections; i ++ ) {
 		nwrite = write( fd, &axel->conn[i].currentbyte, sizeof( axel->conn[i].currentbyte ) );
 		assert( nwrite == sizeof( axel->conn[i].currentbyte ) );
+		nwrite = write( fd, &axel->conn[i].lastbyte, sizeof( axel->conn[i].lastbyte ) );
+		assert( nwrite == sizeof( axel->conn[i].lastbyte ) );
 	}
 	close( fd );
 }
