@@ -5,6 +5,7 @@
   Copyright 2008      Y Giridhar Appaji Nag
   Copyright 2008-2010 Philipp Hagemeister
   Copyright 2015-2016 Joao Eriberto Mota Filho
+  Copyright 2016      Denis Denisov
   Copyright 2016      Mridul Malpotra
   Copyright 2016      Stephen Thirlwall
 
@@ -13,6 +14,20 @@
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of portions of this program with the
+  OpenSSL library under certain conditions as described in each
+  individual source file, and distribute linked combinations including
+  the two.
+
+  You must obey the GNU General Public License in all respects for all
+  of the code used other than OpenSSL. If you modify file(s) with this
+  exception, you may extend this exception to your version of the
+  file(s), but you are not obligated to do so. If you do not wish to do
+  so, delete this exception statement from your version. If you delete
+  this exception statement from all source files in the program, then
+  also delete it here.
+
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -55,6 +70,7 @@ static struct option axel_options[] =
 	{ "help",		0,	NULL,	'h' },
 	{ "version",		0,	NULL,	'V' },
 	{ "alternate",		0,	NULL,	'a' },
+	{ "insecure", 		0,	NULL,	'k' },
 	{ "header",		1,	NULL,	'H' },
 	{ "user-agent",		1,	NULL,	'U' },
 	{ NULL,			0,	NULL,	0 }
@@ -92,7 +108,7 @@ int main( int argc, char *argv[] )
 	{
 		int option;
 
-		option = getopt_long( argc, argv, "s:n:o:S::NqvhVaH:U:", axel_options, NULL );
+		option = getopt_long( argc, argv, "s:n:o:S::NqvhVakH:U:", axel_options, NULL );
 		if( option == -1 )
 			break;
 
@@ -133,6 +149,9 @@ int main( int argc, char *argv[] )
 		case 'a':
 			conf->alternate_output = 1;
 			break;
+		case 'k':
+			conf->insecure = 1;
+			break;
 		case 'N':
 			*conf->http_proxy = 0;
 			break;
@@ -172,6 +191,10 @@ int main( int argc, char *argv[] )
 		return( 1 );
 	}
 
+#ifdef HAVE_OPENSSL
+	ssl_init( conf );
+#endif /* HAVE_OPENSSL */
+
 	if( argc - optind == 0 )
 	{
 		print_help();
@@ -492,32 +515,33 @@ static void print_alternate_output(axel_t *axel)
 {
 	long long int done=axel->bytes_done;
 	long long int total=axel->size;
-	int i,j=0;
+	int i,j=0,offset,end;
 	double now = gettime();
 	int width = get_term_width() - 30;
+	char progress[width+1];
 
-	printf("\r[%3ld%%] [", min(100,(long)(done*100./total+.5) ) );
+	for(i=0;i<width;i++)
+		progress[i] = '.';
 
 	for(i=0;i<axel->conf->num_connections;i++)
 	{
-		for(;j<((double)axel->conn[i].currentbyte/(total+1)*width)-1;j++)
-			putchar('.');
+		offset = ((double)(axel->conn[i].currentbyte)/(total+1)*(width+1));
 
 		if(axel->conn[i].currentbyte<axel->conn[i].lastbyte)
 		{
 			if(now <= axel->conn[i].last_transfer + axel->conf->connection_timeout/2 )
-				putchar(i+'0');
+				progress[offset] = i+'0';
 			else
-				putchar('#');
-		} else 
-			putchar('.');
-
-		j++;
-
-		for(;j<((double)axel->conn[i].lastbyte/(total+1)*width);j++)
-			putchar(' ');
+				progress[offset] = '#';
+		}
+		end = ((double)(axel->conn[i].lastbyte)/(total+1)*(width+1));
+		for(j=offset+1;j<end;j++)
+			progress[j] = ' ';
 	}
 
+	progress[width] = '\0';
+	printf("\r[%3ld%%] [%s", min(100,(long)(done*100./total+.5) ), progress );
+
 	if(axel->bytes_per_second > 1048576)
 		printf( "] [%6.1fMB/s]", (double) axel->bytes_per_second / (1024*1024) );
 	else if(axel->bytes_per_second > 1024)
@@ -563,6 +587,7 @@ void print_help()
 		"-H x\tAdd header string\n"
 		"-U x\tSet user agent\n"
 		"-N\tJust don't use any proxy server\n"
+		"-k\tDon't verify the SSL certificate\n"
 		"-q\tLeave stdout alone\n"
 		"-v\tMore status information\n"
 		"-a\tAlternate progress indicator\n"
@@ -580,6 +605,7 @@ void print_help()
 		"--header=x\t\t-H x\tAdd header string\n"
 		"--user-agent=x\t\t-U x\tSet user agent\n"
 		"--no-proxy\t\t-N\tJust don't use any proxy server\n"
+		"--insecure\t\t-k\tDon't verify the SSL certificate\n"
 		"--quiet\t\t\t-q\tLeave stdout alone\n"
 		"--verbose\t\t-v\tMore status information\n"
 		"--alternate\t\t-a\tAlternate progress indicator\n"
@@ -594,8 +620,10 @@ void print_version()
 {
 	printf( _("\nAxel version %s (%s)\n"), AXEL_VERSION_STRING, ARCH );
 	printf( "\nCopyright 2001-2007 Wilmer van der Gaast," );
+	printf( "\n          2007-2009 Giridhar Appaji Nag," );
+	printf( "\n          2008-2010 Philipp Hagemeister," );
 	printf( "\n          2015-2016 Joao Eriberto Mota Filho," );
-	printf( "\n               2016 Stephen Thirlwall," );
+	printf( "\n          2016      Stephen Thirlwall," );
 	printf( _("\n                    and others." ));
 	printf ( _("\nPlease, see the CREDITS file.\n\n") );
 }
